@@ -1,8 +1,7 @@
 # (c) 2013, Jan-Piet Mens <jpmens(at)gmail.com>
 # (c) 2017 Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 DOCUMENTATION = r"""
     name: csvfile
@@ -18,6 +17,11 @@ DOCUMENTATION = r"""
       col:
         description:  column to return (0 indexed).
         default: "1"
+      keycol:
+        description:  column to search in (0 indexed).
+        default: 0
+        type: int
+        version_added: "2.17"
       default:
         description: what to return if the value is not found in the file.
       delimiter:
@@ -100,7 +104,7 @@ from ansible.module_utils.common.text.converters import to_bytes, to_native, to_
 
 class CSVRecoder:
     """
-    Iterator that reads an encoded stream and reencodes the input to UTF-8
+    Iterator that reads an encoded stream and encodes the input to UTF-8
     """
     def __init__(self, f, encoding='utf-8'):
         self.reader = codecs.getreader(encoding)(f)
@@ -140,14 +144,14 @@ class CSVReader:
 
 class LookupModule(LookupBase):
 
-    def read_csv(self, filename, key, delimiter, encoding='utf-8', dflt=None, col=1):
+    def read_csv(self, filename, key, delimiter, encoding='utf-8', dflt=None, col=1, keycol=0):
 
         try:
             f = open(to_bytes(filename), 'rb')
             creader = CSVReader(f, delimiter=to_native(delimiter), encoding=encoding)
 
             for row in creader:
-                if len(row) and row[0] == key:
+                if len(row) and row[keycol] == key:
                     return row[int(col)]
         except Exception as e:
             raise AnsibleError("csvfile: %s" % to_native(e))
@@ -176,6 +180,7 @@ class LookupModule(LookupBase):
 
             # parameters override per term using k/v
             try:
+                reset_params = False
                 for name, value in kv.items():
                     if name == '_raw_params':
                         continue
@@ -183,7 +188,11 @@ class LookupModule(LookupBase):
                         raise AnsibleAssertionError('%s is not a valid option' % name)
 
                     self._deprecate_inline_kv()
-                    paramvals[name] = value
+                    self.set_option(name, value)
+                    reset_params = True
+
+                if reset_params:
+                    paramvals = self.get_options()
 
             except (ValueError, AssertionError) as e:
                 raise AnsibleError(e)
@@ -193,7 +202,7 @@ class LookupModule(LookupBase):
                 paramvals['delimiter'] = "\t"
 
             lookupfile = self.find_file_in_search_path(variables, 'files', paramvals['file'])
-            var = self.read_csv(lookupfile, key, paramvals['delimiter'], paramvals['encoding'], paramvals['default'], paramvals['col'])
+            var = self.read_csv(lookupfile, key, paramvals['delimiter'], paramvals['encoding'], paramvals['default'], paramvals['col'], paramvals['keycol'])
             if var is not None:
                 if isinstance(var, MutableSequence):
                     for v in var:
