@@ -14,9 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
-# Make coding more python3-ish
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 DOCUMENTATION = '''
     name: linear
@@ -33,7 +31,6 @@ DOCUMENTATION = '''
 
 from ansible import constants as C
 from ansible.errors import AnsibleError, AnsibleAssertionError, AnsibleParserError
-from ansible.executor.play_iterator import IteratingStates, FailedStates
 from ansible.module_utils.common.text.converters import to_text
 from ansible.playbook.handler import Handler
 from ansible.playbook.included_file import IncludedFile
@@ -48,12 +45,6 @@ display = Display()
 
 class StrategyModule(StrategyBase):
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-        # used for the lockstep to indicate to run handlers
-        self._in_handlers = False
-
     def _get_next_task_lockstep(self, hosts, iterator):
         '''
         Returns a list of (host, task) tuples, where the task may
@@ -75,52 +66,35 @@ class StrategyModule(StrategyBase):
         if not state_task_per_host:
             return [(h, None) for h in hosts]
 
-        if self._in_handlers and not any(filter(
-            lambda rs: rs == IteratingStates.HANDLERS,
-            (s.run_state for s, dummy in state_task_per_host.values()))
-        ):
-            self._in_handlers = False
-
-        if self._in_handlers:
-            lowest_cur_handler = min(
-                s.cur_handlers_task for s, t in state_task_per_host.values()
-                if s.run_state == IteratingStates.HANDLERS
-            )
-        else:
-            task_uuids = [t._uuid for s, t in state_task_per_host.values()]
-            _loop_cnt = 0
-            while _loop_cnt <= 1:
-                try:
-                    cur_task = iterator.all_tasks[iterator.cur_task]
-                except IndexError:
-                    # pick up any tasks left after clear_host_errors
-                    iterator.cur_task = 0
-                    _loop_cnt += 1
-                else:
-                    iterator.cur_task += 1
-                    if cur_task._uuid in task_uuids:
-                        break
+        task_uuids = {t._uuid for s, t in state_task_per_host.values()}
+        _loop_cnt = 0
+        while _loop_cnt <= 1:
+            try:
+                cur_task = iterator.all_tasks[iterator.cur_task]
+            except IndexError:
+                # pick up any tasks left after clear_host_errors
+                iterator.cur_task = 0
+                _loop_cnt += 1
             else:
-                # prevent infinite loop
-                raise AnsibleAssertionError(
-                    'BUG: There seems to be a mismatch between tasks in PlayIterator and HostStates.'
-                )
+                iterator.cur_task += 1
+                if cur_task._uuid in task_uuids:
+                    break
+        else:
+            # prevent infinite loop
+            raise AnsibleAssertionError(
+                'BUG: There seems to be a mismatch between tasks in PlayIterator and HostStates.'
+            )
 
         host_tasks = []
         for host, (state, task) in state_task_per_host.items():
-            if ((self._in_handlers and lowest_cur_handler == state.cur_handlers_task) or
-                    (not self._in_handlers and cur_task._uuid == task._uuid)):
+            if cur_task._uuid == task._uuid:
                 iterator.set_state_for_host(host.name, state)
                 host_tasks.append((host, task))
             else:
                 host_tasks.append((host, noop_task))
 
-        # once hosts synchronize on 'flush_handlers' lockstep enters
-        # '_in_handlers' phase where handlers are run instead of tasks
-        # until at least one host is in IteratingStates.HANDLERS
-        if (not self._in_handlers and cur_task.action in C._ACTION_META and
-                cur_task.args.get('_raw_params') == 'flush_handlers'):
-            self._in_handlers = True
+        if cur_task.action in C._ACTION_META and cur_task.args.get('_raw_params') == 'flush_handlers':
+            iterator.all_tasks[iterator.cur_task:iterator.cur_task] = [h for b in iterator._play.handlers for h in b.block]
 
         return host_tasks
 
@@ -213,30 +187,21 @@ class StrategyModule(StrategyBase):
                                 skip_rest = True
                                 break
 
-                        run_once = templar.template(task.run_once) or action and getattr(action, 'BYPASS_HOST_LOOP', False)
+                        run_once = action and getattr(action, 'BYPASS_HOST_LOOP', False) or templar.template(task.run_once)
+                        try:
+                            task.name = to_text(templar.template(task.name, fail_on_undefined=False), nonstring='empty')
+                        except Exception as e:
+                            display.debug(f"Failed to templalte task name ({task.name}), ignoring error and continuing: {e}")
 
                         if (task.any_errors_fatal or run_once) and not task.ignore_errors:
                             any_errors_fatal = True
 
                         if not callback_sent:
-                            display.debug("sending task start callback, copying the task so we can template it temporarily")
-                            saved_name = task.name
-                            display.debug("done copying, going to template now")
-                            try:
-                                task.name = to_text(templar.template(task.name, fail_on_undefined=False), nonstring='empty')
-                                display.debug("done templating")
-                            except Exception:
-                                # just ignore any errors during task name templating,
-                                # we don't care if it just shows the raw name
-                                display.debug("templating failed for some reason")
-                            display.debug("here goes the callback...")
                             if isinstance(task, Handler):
                                 self._tqm.send_callback('v2_playbook_on_handler_task_start', task)
                             else:
                                 self._tqm.send_callback('v2_playbook_on_task_start', task, is_conditional=False)
-                            task.name = saved_name
                             callback_sent = True
-                            display.debug("sending task start callback")
 
                         self._blocked_hosts[host.get_name()] = True
                         self._queue_task(host, task, task_vars, play_context)
@@ -293,7 +258,12 @@ class StrategyModule(StrategyBase):
                                 )
                             else:
                                 is_handler = isinstance(included_file._task, Handler)
-                                new_blocks = self._load_included_file(included_file, iterator=iterator, is_handler=is_handler)
+                                new_blocks = self._load_included_file(
+                                    included_file,
+                                    iterator=iterator,
+                                    is_handler=is_handler,
+                                    handle_stats_and_callbacks=False,
+                                )
 
                             # let PlayIterator know about any new handlers included via include_role or
                             # import_role within include_role/include_taks
@@ -316,7 +286,7 @@ class StrategyModule(StrategyBase):
                                     final_block = new_block.filter_tagged_tasks(task_vars)
                                     display.debug("done filtering new block on tags")
 
-                                    included_tasks.extend(final_block.get_tasks())
+                                included_tasks.extend(final_block.get_tasks())
 
                                 for host in hosts_left:
                                     if host in included_file._hosts:
@@ -326,13 +296,19 @@ class StrategyModule(StrategyBase):
                         except AnsibleParserError:
                             raise
                         except AnsibleError as e:
-                            if included_file._is_role:
-                                # include_role does not have on_include callback so display the error
-                                display.error(to_text(e), wrap_text=False)
+                            display.error(to_text(e), wrap_text=False)
                             for r in included_file._results:
                                 r._result['failed'] = True
+                                r._result['reason'] = str(e)
+                                self._tqm._stats.increment('failures', r._host.name)
+                                self._tqm.send_callback('v2_runner_on_failed', r)
                                 failed_includes_hosts.add(r._host)
-                            continue
+                        else:
+                            # since we skip incrementing the stats when the task result is
+                            # first processed, we do so now for each host in the list
+                            for host in included_file._hosts:
+                                self._tqm._stats.increment('ok', host.name)
+                            self._tqm.send_callback('v2_playbook_on_include', included_file)
 
                     for host in failed_includes_hosts:
                         self._tqm._failed_hosts[host.name] = True
@@ -356,25 +332,16 @@ class StrategyModule(StrategyBase):
                 failed_hosts = []
                 unreachable_hosts = []
                 for res in results:
-                    # execute_meta() does not set 'failed' in the TaskResult
-                    # so we skip checking it with the meta tasks and look just at the iterator
-                    if (res.is_failed() or res._task.action in C._ACTION_META) and iterator.is_failed(res._host):
+                    if res.is_failed():
                         failed_hosts.append(res._host.name)
                     elif res.is_unreachable():
                         unreachable_hosts.append(res._host.name)
 
-                # if any_errors_fatal and we had an error, mark all hosts as failed
-                if any_errors_fatal and (len(failed_hosts) > 0 or len(unreachable_hosts) > 0):
-                    dont_fail_states = frozenset([IteratingStates.RESCUE, IteratingStates.ALWAYS])
+                if any_errors_fatal and (failed_hosts or unreachable_hosts):
                     for host in hosts_left:
-                        (s, dummy) = iterator.get_next_task_for_host(host, peek=True)
-                        # the state may actually be in a child state, use the get_active_state()
-                        # method in the iterator to figure out the true active state
-                        s = iterator.get_active_state(s)
-                        if s.run_state not in dont_fail_states or \
-                           s.run_state == IteratingStates.RESCUE and s.fail_state & FailedStates.RESCUE != 0:
+                        if host.name not in failed_hosts:
                             self._tqm._failed_hosts[host.name] = True
-                            result |= self._tqm.RUN_FAILED_BREAK_PLAY
+                            iterator.mark_host_failed(host)
                 display.debug("done checking for any_errors_fatal")
 
                 display.debug("checking for max_fail_percentage")
