@@ -2,12 +2,9 @@
 
 # NOTE: This file resides in the _util/target directory to ensure compatibility with all supported Python versions.
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 REMOTE_ONLY_PYTHON_VERSIONS = (
-    '2.7',
-    '3.6',
     '3.7',
     '3.8',
     '3.9',
@@ -17,6 +14,4 @@ CONTROLLER_PYTHON_VERSIONS = (
     '3.10',
     '3.11',
     '3.12',
-    '3.13',
-    '3.14',
 )
